import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class PanningAndZooming extends TiltingGraphics {
    private static final long serialVersionUID = 1L;

    private static final double zoomFactor = 1.1;

    /*
     * Translation offsets and scaling factor to be applied before rendering to
     * implement panning and zooming behavior.
     */
    private double translateX = 0;

    private double translateY = 0;

    private double scale = 1;

    /*
     * Last mouse coordinates.
     */
    private int lastX;

    private int lastY;

    /**
     * Constructs the component which we'll use like a canvas for our own custom
     * drawing.
     */
    public PanningAndZooming() {
        super();

        /*
         * Register mouse and keyboard listeners.
         */
        this.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                lastX = e.getX();
                lastY = e.getY();
            }
        });
        this.addMouseMotionListener(new MouseMotionAdapter() {
            @Override
            public void mouseDragged(MouseEvent e) {
                int diffX = e.getX() - lastX;
                int diffY = e.getY() - lastY;

                panBy(diffX, diffY);

                lastX = e.getX();
                lastY = e.getY();
            }

            @Override
            public void mouseMoved(MouseEvent e) {
                lastX = e.getX();
                lastY = e.getY();
            }
        });
        this.addMouseWheelListener(new MouseWheelListener() {
            public void mouseWheelMoved(MouseWheelEvent e) {
                zoomAt(e.getX(), e.getY(), -e.getWheelRotation());
            }
        });
        this.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                switch (e.getKeyCode()) {
                case KeyEvent.VK_EQUALS:
                case KeyEvent.VK_PLUS:
                    zoomAt(lastX, lastY, 1);
                    break;
                case KeyEvent.VK_MINUS:
                    zoomAt(lastX, lastY, -1);
                    break;
                case KeyEvent.VK_0:
                    reset();
                    break;
                }
            }
        });

        /*
         * Make sure we have focus so we can get keyboard events.
         */
        this.setFocusable(true);
        this.requestFocus();
    }

    /**
     * Resets translation offsets to zeros and scaling factor to 1.
     */
    protected void reset() {
        translateX = 0;
        translateY = 0;
        scale = 1;
        repaint();
    }

    /**
     * Pans the view by diffX and diffY (in pixels).
     * 
     * @param diffX
     * @param diffY
     */
    protected void panBy(int diffX, int diffY) {
        //
        // YOUR CODE HERE
        //

        repaint();
    }

    /**
     * Zooms the view by zoomFactor^zoom centered around (x,y). x and y are in
     * pixels.
     * 
     * @param x
     * @param y
     * @param zoom
     */
    protected void zoomAt(int x, int y, int zoom) {
        double factor = Math.pow(zoomFactor, zoom);

        //
        // YOUR CODE HERE
        //

        repaint();
    }

    @Override
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;

        /*
         * Set up the transformation of the coordinate system before
         * getting the parent class to paint.
         */
        g2d.translate(translateX, translateY);
        g2d.scale(scale, scale);

        super.paintComponent(g2d);
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                JFrame frame = new JFrame("PanningAndZooming");
                frame.add(new PanningAndZooming());
                frame.pack();
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setVisible(true);
            }
        });
    }

}
